﻿Imports System.Windows.Controls.Primitives
Imports Infragistics.Windows.Editors
Imports Infragistics.Controls.Editors
Imports Infragistics.Controls.Editors.Primitives
Imports Infragistics.Windows
Imports System.ComponentModel

Namespace DhpCustomControls

	<TemplatePart(Name:="PART_DhpMultiColumnComboEditorBase_TextBlock", Type:=GetType(TextBlock))>
	<TemplatePart(Name:="PART_DhpMultiColumnComboEditorBase_ComboEditor", Type:=GetType(XamMultiColumnComboEditor))>
	Public Class DhpMultiColumnComboEditorBase
		Inherits XamTextEditor

#Region "Static Constructor"

		Shared Sub New()

			DefaultStyleKeyProperty.OverrideMetadata(GetType(DhpMultiColumnComboEditorBase),
																							 New FrameworkPropertyMetadata(GetType(DhpMultiColumnComboEditorBase)))

			ItemsSourceProperty = DependencyProperty.Register("ItemsSource",
																												GetType(IEnumerable),
																												GetType(DhpMultiColumnComboEditorBase),
																												New FrameworkPropertyMetadata(Nothing,
																													New PropertyChangedCallback(AddressOf OnItemsSourceChanged),
																													New CoerceValueCallback(AddressOf OnCoerceItemsSource)))

			ItemsSourceChangedEvent = EventManager.RegisterRoutedEvent("ItemsSourceChanged",
																																 RoutingStrategy.Bubble,
																																 GetType(RoutedEventHandler),
																																 GetType(DhpMultiColumnComboEditorBase))

			CustomValueEnteredActionProperty = DependencyProperty.Register("CustomValueEnteredAction",
																																		 GetType(CustomValueEnteredActions),
																																		 GetType(DhpMultiColumnComboEditorBase),
																																		 New FrameworkPropertyMetadata(Nothing))

			ColumnsProperty = DependencyProperty.Register("Columns",
																										GetType(ComboColumnCollection),
																										GetType(DhpMultiColumnComboEditorBase),
																										New FrameworkPropertyMetadata(Nothing))

			DropDownClosedEvent = EventManager.RegisterRoutedEvent( _
					"DropDownClosed", RoutingStrategy.Bubble, _
					GetType(RoutedEventHandler), _
					GetType(DhpMultiColumnComboEditorBase))

		End Sub

#End Region

#Region "Fields"

		Private _multiColumnComboEditor As XamMultiColumnComboEditor

#End Region

#Region "ItemsSource Property"

		Public Shared ReadOnly ItemsSourceProperty As DependencyProperty

		Public Property ItemsSource() As IEnumerable
			Get
				Return DirectCast(GetValue(ItemsSourceProperty), IEnumerable)
			End Get
			Set(value As IEnumerable)
				SetValue(ItemsSourceProperty, value)
			End Set
		End Property

		Private Shared Sub OnItemsSourceChanged(o As DependencyObject, e As DependencyPropertyChangedEventArgs)
			Dim editor As DhpMultiColumnComboEditorBase = TryCast(o, DhpMultiColumnComboEditorBase)
			If editor IsNot Nothing Then
				editor.OnItemsSourceChanged(DirectCast(e.OldValue, IEnumerable), DirectCast(e.NewValue, IEnumerable))
			End If
		End Sub

		Protected Overridable Sub OnItemsSourceChanged(oldValue As IEnumerable, newValue As IEnumerable)
			Me.RaiseEvent(New RoutedEventArgs(DhpMultiColumnComboEditorBase.ItemsSourceChangedEvent, Me))
		End Sub

		Private Shared Function OnCoerceItemsSource(o As DependencyObject, value As Object) As Object
			Dim editor As DhpMultiColumnComboEditorBase = TryCast(o, DhpMultiColumnComboEditorBase)
			If editor IsNot Nothing Then
				Return editor.OnCoerceItemsSource(DirectCast(value, IEnumerable))
			Else
				Return value
			End If
		End Function

		Protected Overridable Function OnCoerceItemsSource(value As IEnumerable) As IEnumerable
			Return value
		End Function

#End Region

#Region "ItemsSourceChanged Event"

		Public Shared ReadOnly ItemsSourceChangedEvent As RoutedEvent

		Public Custom Event ItemsSourceChanged As RoutedEventHandler
			' http://msdn.microsoft.com/en-us/library/ms752288.aspx
			AddHandler(ByVal value As RoutedEventHandler)
				Me.AddHandler(ItemsSourceChangedEvent, value)
			End AddHandler
			RemoveHandler(ByVal value As RoutedEventHandler)
				Me.RemoveHandler(ItemsSourceChangedEvent, value)
			End RemoveHandler
			RaiseEvent(ByVal sender As Object, ByVal e As RoutedEventArgs)
				Me.RaiseEvent(e)
			End RaiseEvent
		End Event

#End Region

#Region "CustomValueEnteredAction Property"

		Public Shared ReadOnly CustomValueEnteredActionProperty As DependencyProperty

		Public Property CustomValueEnteredAction() As CustomValueEnteredActions
			Get
				Return DirectCast(GetValue(CustomValueEnteredActionProperty), CustomValueEnteredActions)
			End Get
			Set(value As CustomValueEnteredActions)
				SetValue(CustomValueEnteredActionProperty, value)
			End Set
		End Property

#End Region

#Region "Columns Property"

		Public Shared ReadOnly ColumnsProperty As DependencyProperty

		Public Property Columns() As ComboColumnCollection
			Get
				Return DirectCast(GetValue(ColumnsProperty), ComboColumnCollection)
			End Get
			Set(value As ComboColumnCollection)
				SetValue(ColumnsProperty, value)

				'Not sure if this works.
				'Wasn't able to test it!

				If _multiColumnComboEditor IsNot Nothing Then

					_multiColumnComboEditor.Columns.Clear()

					For Each comboColumn As ComboColumn In value
						_multiColumnComboEditor.Columns.Add(comboColumn)
					Next

				End If

			End Set
		End Property

#End Region

#Region "Events"

		Public Overrides Sub OnApplyTemplate()
			MyBase.OnApplyTemplate()

			If IsInEditMode Then
				_multiColumnComboEditor = TryCast(MyBase.GetTemplateChild("PART_DhpMultiColumnComboEditorBase_ComboEditor"), XamMultiColumnComboEditor)
				If _multiColumnComboEditor IsNot Nothing Then
					AddHandler _multiColumnComboEditor.Loaded, AddressOf XamMultiColumnComboEditor_Loaded
					AddHandler _multiColumnComboEditor.DropDownOpening, AddressOf XamMultiColumnComboEditor_DropDownOpening
					AddHandler _multiColumnComboEditor.DropDownOpened, AddressOf XamMultiColumnComboEditor_DropDownOpened
					AddHandler _multiColumnComboEditor.DropDownClosed, AddressOf XamMultiColumnComboEditor_DropDownClosed
				End If
			Else
				Dim multiColumnComboTextBlock As TextBlock = TryCast(MyBase.GetTemplateChild("PART_DhpMultiColumnComboEditorBase_TextBlock"), TextBlock)
				'TextBlock events here
				If multiColumnComboTextBlock IsNot Nothing Then

				End If
			End If
		End Sub

		Private Sub XamMultiColumnComboEditor_Loaded(sender As Object, e As RoutedEventArgs)
			TryCast(sender, XamMultiColumnComboEditor).IsDropDownOpen = True
			Dim txt As SpecializedTextBox = TryCast(Utilities.GetDescendantFromType(TryCast(sender, DependencyObject), GetType(SpecializedTextBox), False), SpecializedTextBox)
			txt.SelectAll()
			txt.Focus()
		End Sub

		Private Sub XamMultiColumnComboEditor_DropDownOpening(sender As Object, e As CancelEventArgs)
			e.Cancel = (Not Me.IsEnabled)
		End Sub

		Private Sub XamMultiColumnComboEditor_DropDownOpened(sender As Object, e As EventArgs)
			Try
				Dim grid As Grid = TryCast(VisualTreeHelper.GetChild(TryCast(sender, Infragistics.Controls.Editors.XamMultiColumnComboEditor), 0), Grid)
				Dim popup As Popup = TryCast(TryCast(TryCast(grid.Children(0), Border).Child, Grid).Children(0), Popup)

				TryCast(TryCast(popup.Child, Grid).Children(0), Border).Width = 170
			Catch exec As System.Exception
			End Try
		End Sub

		Public Shared ReadOnly DropDownClosedEvent As RoutedEvent

		Public Custom Event DropDownClosed As RoutedEventHandler
			AddHandler(ByVal value As RoutedEventHandler)
				Me.AddHandler(DropDownClosedEvent, value)
			End AddHandler
			RemoveHandler(ByVal value As RoutedEventHandler)
				Me.RemoveHandler(DropDownClosedEvent, value)
			End RemoveHandler
			RaiseEvent(ByVal sender As Object, ByVal e As RoutedEventArgs)
				Me.RaiseEvent(e)
			End RaiseEvent
		End Event

		Private Sub XamMultiColumnComboEditor_DropDownClosed(sender As Object, e As EventArgs)
			'EndEditMode(True, True)
			Dim args As New RoutedEventArgs()
			args.RoutedEvent = DropDownClosedEvent
			Me.RaiseEvent(args)
		End Sub

#End Region

	End Class

End Namespace